package net.pingex.dcf.commands.audit;

import java.util.Map;

/**
 * This class contains results for an audit.
 */
public class AuditResult
{
	/**
	 * Result code for this audit
	 */
	private ResultCode opCode;

	/**
	 * Details message about what happened. Mostly used with FAIL and WARN opcodes.
	 */
	private String message;

	/**
	 * Sub audits.
	 */
	private Map<IAuditComponentProvider, AuditResult> subAuditsResults;

	/**
	 * Full constructor, with sub audits.
	 * @param opCode The result code of the operation.
	 * @param message Optional message, pass `null` for no message.
	 * @param subAuditsResults A result for each sub audit.
	 */
	public AuditResult(ResultCode opCode, String message, Map<IAuditComponentProvider, AuditResult> subAuditsResults)
	{
		this.opCode = opCode;
		this.message = message;
		this.subAuditsResults = subAuditsResults;
	}

	/**
	 * Constructor without sub audits.
	 * @param opCode The result code of the operation.
	 * @param message Optional message, pass `null` for no message.
	 */
	public AuditResult(ResultCode opCode, String message)
	{
		this.opCode = opCode;
		this.message = message;
	}

	/**
	 * Result code for this audit
	 */
	public ResultCode getOpCode()
	{
		return opCode;
	}

	/**
	 * Details message about what happened. Mostly used with FAIL and WARN opcodes.
	 */
	public String getMessage()
	{
		return message;
	}

	/**
	 * Sub audits.
	 * @return A map of these subaudits.
	 * @throws UnsupportedOperationException in case there is not sub audits.
	 */
	public Map<IAuditComponentProvider, AuditResult> getSubAuditsResults() throws UnsupportedOperationException
	{
		if(subAuditsResults != null)
			return subAuditsResults;
		else throw new UnsupportedOperationException();
	}

	/**
	 * All the possible result codes for an andit and its component.
	 */
	public enum ResultCode
	{
		/**
		 * The test passed without any issue.
		 */
		PASS,

		/**
		 * The audit failed.
		 */
		FAIL,

		/**
		 * The test passed with a warning (contained in the message field)
		 */
		WARN,

		/**
		 * Test has been ignored.
		 */
		NOOP
	}
}
